#pragma once

#include <chrono>
#include <string>
#include <memory>
#include <cppamqp/types.h>
#include <cppamqp/config.h>
#include <cppamqp/channel.h>
#include <cppamqp/consume_result.h>

namespace cppamqp
{

    struct connection
    {
    public:
        static inline const std::string&               default_vhost           ();
        static inline const std::string&               default_username        ();
        static inline const std::string&               default_password        ();
        static inline const std::chrono::milliseconds& default_consume_timeout ();

    private:
        struct internal
        {
            amqp_connection_state_t connection;
            int                     reply_code;
            bool                    auto_close { false };

            internal();
            ~internal();
        };

    private:
        std::unique_ptr<internal> _internal;

    public:
        inline operator bool() const;

        inline amqp_connection_state_t&        handle      ();
        inline const amqp_connection_state_t&  handle      () const;

        void tcp_connect(
            const std::string& hostname,
            uint port);

        void login_plain(
            const std::string&  p_username        = default_username(),
            const std::string&  p_password        = default_password(),
            const std::string&  p_vhost           = default_vhost(),
            int                 p_heartbeat       = AMQP_DEFAULT_HEARTBEAT,
            int                 p_max_channels    = AMQP_DEFAULT_MAX_CHANNELS,
            int                 p_max_frame_size  = AMQP_DEFAULT_FRAME_SIZE);

        void login_external(
            const std::string&  p_identify,
            const std::string&  p_vhost           = default_vhost(),
            int                 p_heartbeat       = AMQP_DEFAULT_HEARTBEAT,
            int                 p_max_channels    = AMQP_DEFAULT_MAX_CHANNELS,
            int                 p_max_frame_size  = AMQP_DEFAULT_FRAME_SIZE);

        channel        open_channel   (channel_number p_channel);
        consume_result consume_message(const std::chrono::milliseconds& p_timeout = default_consume_timeout());
        void           close          (int p_status = AMQP_REPLY_SUCCESS, bool p_force = false);
    };

}