#pragma once

#include <cppamqp/enums.h>
#include <cppamqp/helper.h>
#include <cppamqp/connection.h>
#include <cppamqp/consume_result.h>

namespace cppamqp
{

    consume_result::consume_result(const connection& p_connection, consume_result_type p_type)
        : _connection   (p_connection)
        , type          (p_type)
        { }

    consume_result::consume_result(const connection& p_connection, consume_result_type p_type, const amqp_message_t& p_message)
        : _connection   (p_connection)
        , type          (p_type)
        , message       (p_message)
        { }

    consume_result::consume_result(const connection& p_connection, const amqp_envelope_t& p_envelope)
        : _connection   (p_connection)
        , type          (consume_result_type::success)
        , message       (p_envelope.message)
        , channel       (p_envelope.channel)
        , consumer_tag  (__impl::from_bytes(p_envelope.consumer_tag))
        , delivery_tag  (p_envelope.delivery_tag)
        , redelivered   (p_envelope.redelivered)
        , exchange      (__impl::from_bytes(p_envelope.exchange))
        , routing_key   (__impl::from_bytes(p_envelope.routing_key))
        { }

    void consume_result::ack()
    {
        __impl::check_and_raise(
            amqp_basic_ack(_connection.handle(), channel, delivery_tag, false),
            "error while ack message",
            false);
    }

    void consume_result::nack(bool requeue)
    {
        __impl::check_and_raise(
            amqp_basic_nack(_connection.handle(), channel, delivery_tag, false, requeue),
            "error while nack message",
            false);
    }

}