#include <cppamqp/helper.h>
#include <cppamqp/exception.inl>
#include <cpplogging/interface.h>

void ::cppamqp::__impl::check_and_raise(int returnCode, std::string msg, bool logOnly)
{
    if (returnCode == AMQP_STATUS_OK)
        return;
    if (logOnly)
    {
        if (!msg.empty())
            msg += " - ";
        msg += amqp_error_string2(returnCode);
        cppamqp_log(error) << msg;
    }
    else
        throw cppamqp::exception(msg, static_cast<amqp_status_enum>(returnCode));
}

void ::cppamqp::__impl::check_and_raise(const amqp_rpc_reply_t& reply, std::string msg, bool logOnly)
{
    switch(reply.reply_type)
    {
        case AMQP_RESPONSE_NORMAL:
            return;

        case AMQP_RESPONSE_SERVER_EXCEPTION:
        {
            if (!msg.empty())
                msg += " - ";
            msg += std::string("Server error: ID=") + std::to_string(reply.reply.id);
            if (logOnly) {
                cppamqp_log(error) << msg;
            } else {
                throw cppamqp::exception(msg);
            }
        }
        break;

        case AMQP_RESPONSE_LIBRARY_EXCEPTION:
            check_and_raise(reply.library_error, msg, logOnly);
            break;

        default:
        {
            if (!msg.empty())
                msg += " - ";
            msg += "unknown reply";
            if (logOnly) {
                cppamqp_log(error) << msg;
            } else {
                throw cppamqp::exception(msg);
            }
        }
        break;
    }
}
