#include <cppamqp/helper.inl>
#include <cppamqp/publish_options.h>

using namespace ::cppamqp;

template<class T, class S>
inline T simpleConvert(const S& value)
    { return static_cast<T>(value); }

template<>
inline amqp_bytes_t simpleConvert<amqp_bytes_t, std::string>(const std::string& value)
    { return __impl::make_bytes(value); }

template<class T, class S>
inline void addProperty(amqp_basic_properties_t& properties, amqp_flags_t flag, T amqp_basic_properties_t::*member, const ::cppcore::nullable<S>& value)
{
    if (value)
    {
        properties.*member = simpleConvert<T, S>(value());
        properties._flags |= flag;
    }
}

const amqp_basic_properties_t* publish_options::properties() const
{
    _properties._flags = 0;
    addProperty(_properties, AMQP_BASIC_CONTENT_TYPE_FLAG,      &amqp_basic_properties_t::content_type,     content_type);
    addProperty(_properties, AMQP_BASIC_CONTENT_ENCODING_FLAG,  &amqp_basic_properties_t::content_encoding, content_encoding);
    addProperty(_properties, AMQP_BASIC_DELIVERY_MODE_FLAG,     &amqp_basic_properties_t::delivery_mode,    delivery_mode);
    addProperty(_properties, AMQP_BASIC_PRIORITY_FLAG,          &amqp_basic_properties_t::priority,         priority);
    addProperty(_properties, AMQP_BASIC_CORRELATION_ID_FLAG,    &amqp_basic_properties_t::correlation_id,   correlation_id);
    addProperty(_properties, AMQP_BASIC_REPLY_TO_FLAG,          &amqp_basic_properties_t::reply_to,         reply_to);
    addProperty(_properties, AMQP_BASIC_EXPIRATION_FLAG,        &amqp_basic_properties_t::expiration,       expiration);
    addProperty(_properties, AMQP_BASIC_MESSAGE_ID_FLAG,        &amqp_basic_properties_t::message_id,       message_id);
    addProperty(_properties, AMQP_BASIC_TIMESTAMP_FLAG,         &amqp_basic_properties_t::timestamp,        timestamp);
    addProperty(_properties, AMQP_BASIC_TYPE_FLAG,              &amqp_basic_properties_t::type,             type);
    addProperty(_properties, AMQP_BASIC_USER_ID_FLAG,           &amqp_basic_properties_t::user_id,          user_id);
    addProperty(_properties, AMQP_BASIC_APP_ID_FLAG,            &amqp_basic_properties_t::app_id,           app_id);
    addProperty(_properties, AMQP_BASIC_CLUSTER_ID_FLAG,        &amqp_basic_properties_t::cluster_id,       cluster_id);
    return &_properties;
}

void publish_options::reset()
{
    content_type.reset();
    content_encoding.reset();
    delivery_mode.reset();
    priority.reset();
    correlation_id.reset();
    reply_to.reset();
    expiration.reset();
    message_id.reset();
    timestamp.reset();
    type.reset();
    user_id.reset();
    app_id.reset();
    cluster_id.reset();
}
